#
# Copyright 2012 Double Precision, Inc.
# See COPYING for distribution information.
#

Summary: LIBCXX
Name: libcxxbase
Version: 0.12
Release: 1%{?extrarelease}
License: GPL
URL: http://www.libcxx.org
Group: System Environment/Libraries
Source0: libcxx-%{version}.tar.bz2

%define _hardened_build 1

BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root
BuildRequires: courier-unicode-devel >= 2.0
BuildRequires: gettext
BuildRequires: gnutls-devel
BuildRequires: libgcrypt-devel
BuildRequires: %{__perl}
BuildRequires: libxslt
BuildRequires: pcre-devel
BuildRequires: libyaml-devel
BuildRequires: file-devel
BuildRequires: libidn-devel
BuildRequires: libtool-ltdl-devel
BuildRequires: libxml2-devel
BuildRequires: cups-devel
Requires(post): /bin/systemctl
Requires(preun): /bin/systemctl

%description
The C++ class library provides many classes that are useful
for developing C++ applications on Linux.

%package ltdl
Group: System Environment/Libraries
Summary: libcxx libtool wrapper
Requires: %{name} = 0:%{version}-%{release}

%package cups
Group: System Environment/Libraries
Summary: libcxx CUPS library
Requires: %{name} = 0:%{version}-%{release}

%package devel
Group: Development/Libraries
Summary: libcxx development libraries
Requires: %{name} = 0:%{version}-%{release}
Requires: %{name}-ltdl = 0:%{version}-%{release}
Requires: %{name}-cups = 0:%{version}-%{release}
Requires: %{_datadir}/aclocal
Requires: libxslt
Requires: gnutls-devel
Requires: libgcrypt-devel
Requires: pcre-devel
Requires: libyaml-devel
Requires: file-devel
Requires: libidn-devel
Requires: cups-devel
Requires: courier-unicode-devel

%package httpd
Group: System Environment/Daemons
Summary: Apache httportmap configuration
Requires: %{name} = 0:%{version}-%{release}
Requires: /etc/httpd/conf.d

%description ltdl
This package installs the libcxxltdl wrapper for libtool's ltdl library.

%description cups
This package installs the libcxxcups wrapper for the CUPS framework.

%description devel
This package installs files needed to develop applications that use libcxx.

%description httpd
This package configurates Apache to redirect http://localhost/portmap URLs
to the httportmap daemon. This package must be installed if Apache is also
installed on this server.

%prep
%setup -q -n libcxx-%{version}
%configure

%build
%{__make} %{_smp_mflags}

%install
rm -rf $RPM_BUILD_ROOT
%{__make} install DESTDIR=$RPM_BUILD_ROOT INSTALL_TLSPARAM_SUFFIX=.dist
mkdir -p $RPM_BUILD_ROOT/etc/httpd/conf.d
%{__install} -m 0444 httportmap/apache.conf $RPM_BUILD_ROOT/etc/httpd/conf.d/httportmap.conf
mkdir -p $RPM_BUILD_ROOT/etc/rc.d/init.d
%{__install} -m 0555 packaging/fedora/sysinit $RPM_BUILD_ROOT%{_datadir}/httportmap/sysinit
mkdir -p $RPM_BUILD_ROOT%{_unitdir}
%{__install} -m 0644 packaging/fedora/systemd $RPM_BUILD_ROOT%{_unitdir}/httportmap.service

%{__mkdir_p} $RPM_BUILD_ROOT%{_localstatedir}/run/httportmap.systemd
%{__mkdir_p} $RPM_BUILD_ROOT%{_localstatedir}/run/httportmap.priv

for f in $RPM_BUILD_ROOT%{_localstatedir}/run/httportmap $RPM_BUILD_ROOT%{_localstatedir}/run/httportmap.client $RPM_BUILD_ROOT%{_localstatedir}/run/httportmap.priv/socket
do
	%{__perl} -e 'use Socket; socket(S, PF_UNIX, SOCK_STREAM, 0); bind(S, sockaddr_un("'$f'"));'
done
>$RPM_BUILD_ROOT%{_localstatedir}/run/httportmap.lock
chmod 600 $RPM_BUILD_ROOT%{_localstatedir}/run/httportmap.lock

%{__mkdir_p} $RPM_BUILD_ROOT%{_sysconfdir}/cron.monthly
%{__ln_s} %{_sbindir}/tlsparamsgen.sh $RPM_BUILD_ROOT%{_sysconfdir}/cron.monthly/tlsparamsgen

rm -f $RPM_BUILD_ROOT%{_libdir}/*.la

%clean
rm -rf $RPM_BUILD_ROOT

%post
/sbin/ldconfig
%{_bindir}/properties --set %{_sysconfdir}/httportmapd.properties %{_sbindir}/httportmapd >/dev/null
for f in %{_localstatedir}/tlsparams/system.dh
do
	test -f $f || ln $f.dist $f
done

%systemd_post httportmap.service

if [ $1 -eq 1 ]
then
    /bin/systemctl daemon-reload >/dev/null 2>&1 || :
fi
/bin/systemctl try-reload-or-restart httportmap.service &> /dev/null || :

%preun
%systemd_preun httportmap.service
if test "$1" = 0
then
	rm -f %{_localstatedir}/tlsparams/system.dh
fi

%postun
/sbin/ldconfig

# Do not use systemd_postun_with_restart. This stops httportmap completely.
# try-reload-or-restart in %post is sufficient for our purposes.

%files
%defattr(-,root,root,-)
%{_libdir}/libcxx.so.*
%{_libdir}/libcxxtls.so.*
%{_libdir}/libcxxyaml.so.*
%{_bindir}/*
%{_sbindir}/*
%{_datadir}/aclocal/*
%{_datadir}/httportmap
%{_datadir}/libcxx
%{_mandir}/*/*
%{_unitdir}/*
%attr(0777, root, root) %verify(user group mode) %ghost %config(missingok) %{_localstatedir}/run/httportmap
%attr(0777, root, root) %verify(user group mode) %ghost %config(missingok) %{_localstatedir}/run/httportmap.client
%attr(0700, root, root) %dir %{_localstatedir}/run/httportmap.priv
%attr(0777, root, root) %verify(user group mode) %ghost %config(missingok) %{_localstatedir}/run/httportmap.priv/socket
%attr(0600, root, root) %verify(user group mode) %config(missingok) %{_localstatedir}/run/httportmap.lock
%attr(0700, root, root) %dir %{_localstatedir}/run/httportmap.systemd

%dir %{_sysconfdir}/tlsparams
%config(noreplace) %{_sysconfdir}/tlsparams/*

%{_localstatedir}/tlsparams
%{_sysconfdir}/cron.monthly/*
%config %{_sysconfdir}/*.dat
%config %{_sysconfdir}/*.properties

%files ltdl
%defattr(-,root,root,-)
%{_libdir}/libcxxltdl.so.*

%files cups
%defattr(-,root,root,-)
%{_libdir}/libcxxcups.so.*

%files devel
%defattr(-,root,root,-)
%{_libdir}/*.so
%{_libdir}/*.a
%{_libdir}/pkgconfig/*
%{_includedir}/*
%{_datadir}/libcxx
%{_datadir}/aclocal/*

%files httpd
%defattr(-,root,root,-)
/etc/httpd/conf.d/*

%changelog
* Sat Feb 23 2008 Mr. Sam <sam@email-scan.com> -
- Initial build.
